package net.darkgamer.games.spacehero.activities;

import android.view.KeyEvent;
import javax.microedition.khronos.opengles.GL10;
import net.darkgamer.games.spacehero.ships.PlayerShip;
import org.anddev.andengine.engine.Engine;
import org.anddev.andengine.engine.camera.Camera;
import org.anddev.andengine.engine.options.EngineOptions;
import org.anddev.andengine.engine.options.EngineOptions.ScreenOrientation;
import org.anddev.andengine.engine.options.resolutionpolicy.RatioResolutionPolicy;
import org.anddev.andengine.entity.scene.Scene;
import org.anddev.andengine.entity.scene.background.AutoParallaxBackground;
import org.anddev.andengine.entity.scene.background.ParallaxBackground.ParallaxEntity;
import org.anddev.andengine.entity.scene.menu.MenuScene;
import org.anddev.andengine.entity.scene.menu.MenuScene.IOnMenuItemClickListener;
import org.anddev.andengine.entity.scene.menu.item.IMenuItem;
import org.anddev.andengine.entity.scene.menu.item.SpriteMenuItem;
import org.anddev.andengine.entity.sprite.Sprite;
import org.anddev.andengine.entity.util.FPSLogger;
import org.anddev.andengine.opengl.texture.TextureOptions;
import org.anddev.andengine.opengl.texture.atlas.bitmap.BitmapTextureAtlas;
import org.anddev.andengine.opengl.texture.atlas.bitmap.BitmapTextureAtlasTextureRegionFactory;
import org.anddev.andengine.opengl.texture.region.TextureRegion;
import org.anddev.andengine.ui.activity.BaseGameActivity;

/**
 *
 * @author DkGr
 */
public class SpaceHeroActivity extends BaseGameActivity implements IOnMenuItemClickListener{
    // ===========================================================
    // Constants
    // ===========================================================

    private static final int CAMERA_WIDTH = 480;
    private static final int CAMERA_HEIGHT = 320;

    protected static final int MENU_RESET = 0;
    protected static final int MENU_QUIT = MENU_RESET + 1;

    // ===========================================================
    // Fields
    // ===========================================================

    protected Camera mCamera;

    protected Scene mMainScene;

    private Sprite mBackground;
    private PlayerShip playerShip;
    
    private BitmapTextureAtlas mAutoParallaxBackgroundTexture;
    private BitmapTextureAtlas mMenuTexture;
    
    private TextureRegion mParallaxLayerFront;
    protected TextureRegion mMenuQuitTextureRegion;
    
    protected MenuScene mMenuScene;
    
    

    // ===========================================================
    // Methods for/from SuperClass/Interfaces
    // ===========================================================

    public Engine onLoadEngine() {
        this.mCamera = new Camera(0, 0, CAMERA_WIDTH, CAMERA_HEIGHT);
        return new Engine(new EngineOptions(true, ScreenOrientation.LANDSCAPE, new RatioResolutionPolicy(CAMERA_WIDTH, CAMERA_HEIGHT), this.mCamera));
    }

    public void onLoadResources() {
        BitmapTextureAtlasTextureRegionFactory.setAssetBasePath("gfx/");
        playerShip = new PlayerShip();
        
        this.mAutoParallaxBackgroundTexture = new BitmapTextureAtlas(512, 2048, TextureOptions.DEFAULT);
        this.mParallaxLayerFront = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mAutoParallaxBackgroundTexture, this, "background.png", 0, 0);
        this.mEngine.getTextureManager().loadTextures(this.mAutoParallaxBackgroundTexture);

        this.mMenuTexture = new BitmapTextureAtlas(256, 128, TextureOptions.BILINEAR_PREMULTIPLYALPHA);
        this.mMenuQuitTextureRegion = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mMenuTexture, this, "menu_quit.png", 0, 50);
        this.mEngine.getTextureManager().loadTexture(this.mMenuTexture);
        
        playerShip.setContext(this);
        if(playerShip.loadRessource()){
            this.mEngine.getTextureManager().loadTexture(playerShip.getmShipBitmapTextureAtlas());
            this.mEngine.getTextureManager().loadTexture(playerShip.laser1.getmLaserBitmapTextureAtlas());
            this.mEngine.getTextureManager().loadTexture(playerShip.laser2.getmLaserBitmapTextureAtlas());
            this.mEngine.getTextureManager().loadTexture(playerShip.laser3.getmLaserBitmapTextureAtlas());
            this.mEngine.getTextureManager().loadTexture(playerShip.laser4.getmLaserBitmapTextureAtlas());
        }        
    }

    public Scene onLoadScene() {
        this.mEngine.registerUpdateHandler(new FPSLogger());

        this.createMenuScene();

        this.mMainScene = new Scene();
        final AutoParallaxBackground autoParallaxBackground = new AutoParallaxBackground(0, 0, 0, 10);
        mBackground = new Sprite(0, 0, this.mParallaxLayerFront);
        ParallaxEntity parallaxEntity = new ParallaxEntity(10.0f, mBackground){

            @Override
            public void onDraw(GL10 pGL, float pParallaxValue, Camera pCamera) {
                pGL.glPushMatrix();
                {
                        final float cameraHeight = pCamera.getHeight();
                        final float shapeHeightScaled = mBackground.getHeightScaled();
                        float baseOffset = (pParallaxValue * 10) % shapeHeightScaled;

                        while(baseOffset > 0) {
                                baseOffset -= shapeHeightScaled;
                        }
                        pGL.glTranslatef(0, baseOffset, 0);

                        float currentMaxY = baseOffset;

                        do {
                            mBackground.onDraw(pGL, pCamera);
                            pGL.glTranslatef(0, shapeHeightScaled, 0);
                            currentMaxY += shapeHeightScaled;
                        } while(currentMaxY < cameraHeight);
                }
                pGL.glPopMatrix();
            }
            
        };
        autoParallaxBackground.attachParallaxEntity(parallaxEntity);
        this.mMainScene.setBackground(autoParallaxBackground);

        playerShip.initShip();
        playerShip.setPosition(CAMERA_WIDTH/2, CAMERA_HEIGHT);

        this.mMainScene.attachChild(playerShip.laser1.getLaserSystem());
        this.mMainScene.attachChild(playerShip.laser2.getLaserSystem());
        this.mMainScene.attachChild(playerShip.laser3.getLaserSystem());
        this.mMainScene.attachChild(playerShip.laser4.getLaserSystem());
        this.mMainScene.attachChild(playerShip.getShipSprite());
        this.mMainScene.registerTouchArea(playerShip.getShipSprite());
        this.mMainScene.setTouchAreaBindingEnabled(true);
        
        return this.mMainScene;
    }

    public void onLoadComplete() {
        
    }
    
    @Override
    public boolean onKeyDown(final int pKeyCode, final KeyEvent pEvent) {
            if(pKeyCode == KeyEvent.KEYCODE_MENU && pEvent.getAction() == KeyEvent.ACTION_DOWN) {
                    if(this.mMainScene.hasChildScene()) {
                            /* Remove the menu and reset it. */
                            this.mMenuScene.back();
                    } else {
                            /* Attach the menu. */
                            this.mMainScene.setChildScene(this.mMenuScene, false, true, true);
                    }
                    return true;
            } else {
                    return super.onKeyDown(pKeyCode, pEvent);
            }
    }
    
    public boolean onMenuItemClicked(MenuScene pMenuScene, IMenuItem pMenuItem, float pMenuItemLocalX, float pMenuItemLocalY) {
        switch(pMenuItem.getID()) {
            case MENU_QUIT:
                    /* End Activity. */
                    this.finish();
                    return true;
            default:
                    return false;
        }
    }
    
    
    // ===========================================================
    // Methods
    // ===========================================================
    
    private void createMenuScene() {
        this.mMenuScene = new MenuScene(this.mCamera);

        final SpriteMenuItem quitMenuItem = new SpriteMenuItem(MENU_QUIT, this.mMenuQuitTextureRegion);
        quitMenuItem.setBlendFunction(GL10.GL_SRC_ALPHA, GL10.GL_ONE_MINUS_SRC_ALPHA);
        this.mMenuScene.addMenuItem(quitMenuItem);

        this.mMenuScene.buildAnimations();

        this.mMenuScene.setBackgroundEnabled(false);

        this.mMenuScene.setOnMenuItemClickListener(this);
    }
}